import { minioClient } from '@/app.ts';
import { bucketName } from '@/modules/minio.ts';

type MinioListOpt = {
  prefix: string;
  recursive?: boolean;
};
export type MinioFile = {
  name: string;
  size: number;
  lastModified: Date;
  etag: string;
};
export type MinioDirectory = {
  prefix: string;
  size: number;
};
export type MinioList = (MinioFile | MinioDirectory)[];
export const getMinioList = async (opts: MinioListOpt): Promise<MinioList> => {
  const prefix = opts.prefix;
  const recursive = opts.recursive ?? false;
  return await new Promise((resolve, reject) => {
    let res: any[] = [];
    let hasError = false;
    minioClient
      .listObjectsV2(bucketName, prefix, recursive)
      .on('data', (data) => {
        res.push(data);
      })
      .on('error', (err) => {
        console.error('minio error', opts.prefix, err);
        hasError = true;
      })
      .on('end', () => {
        if (hasError) {
          reject();
          return;
        } else {
          resolve(res);
        }
      });
  });
};
export const getFileStat = async (prefix: string): Promise<any> => {
  try {
    const obj = await minioClient.statObject(bucketName, prefix);
    return obj;
  } catch (e) {
    if (e.code === 'NotFound') {
      return null;
    }
    console.error('get File Stat Error not handle', e);
    return null;
  }
};

export const deleteFile = async (prefix: string): Promise<any> => {
  try {
    await minioClient.removeObject(bucketName, prefix, {
      versionId: 'null',
      forceDelete: true,
    });
    return true;
  } catch (e) {
    console.error('delete File Error not handle', e);
    return false;
  }
};

// 批量删除文件
export const deleteFiles = async (prefixs: string[]): Promise<any> => {
  try {
    await minioClient.removeObjects(bucketName, prefixs);
    return true;
  } catch (e) {
    console.error('delete Files Error not handle', e);
    return false;
  }
};

type GetMinioListAndSetToAppListOpts = {
  username: string;
  appKey: string;
  version: string;
};
// 批量列出文件，并设置到appList的files中
export const getMinioListAndSetToAppList = async (opts: GetMinioListAndSetToAppListOpts) => {
  const { username, appKey, version } = opts;
  const minioList = await getMinioList({ prefix: `${username}/${appKey}/${version}`, recursive: true });
  const files = minioList;
  return files as MinioFile[];
};
